==> nv-wx-srv-eng-1.0.26.tgz: package/index.js <==
const {OD,OASGN,sign_str,add_repr} = require("nv-facutil-basic");
const _url                         = require("url");
const _str                         = require("nv-string-basic");
const _fbsc                        = require("nv-file-basic");
const _S                           = require("nv-server-simple-stream").http.simple;
const _chown_with_name             = require("nv-cli-user").chown_with_name;
const _rrequire                    = require("nv-facutil-rrequire");
const _eng                         = require("./eng");
const _err_bsc                     = require("nv-error-basic");  
const _wxid                        = require("wx-id");
const _wxhb                        = require("wx-hongbao-pay").func; 



class Srv {
   consts  = _eng.DFLT_CONSTS();
   init_consts(usock,nginx_user,wx_app_id_name,wx_app_secret_name) {
       this.consts.usock        = usock;
       this.consts.nginx_user   = nginx_user; 
       this.wx_app_id_name      = wx_app_id_name;
       this.wx_app_secret_name  = wx_app_secret_name; 
   }
   ////-------------------------------------------------
   user  = null;    //user   globalThis.USER
   oss   = null;    //oss    globalThis.OSS
   redis = null;    //redis  globalThis.REDIS
   cfgdb = null;    //cfg    globalThis.CFG
   //
   init_glb(user,oss,redis,cfgdb) {
        this.user  = user;
        this.oss   = oss;
        this.redis = redis;
        this.cfgdb = cfgdb;
   }
   ////----------------------------------------------
   ////---
   appid        = null; //wxappid
   secret       = null;   
   mchid        = null; //mch_id
   mchkey       = null; //mch_key
   api_pem_key  = null; // internal using
   api_pem_cert = null; // internal using
   code2id = (code)=>code;  
   // 
   hongbao_retry_min_intv = 1000;
   hongbao_send_freq = [30,1000];
   //-----------------发送红包函数签名-------------------------------------- 
   send_hb = ({
      re_openid,               //前端 通过接口get_openid({code}) 获取 openid, 注意这里 openid 名需要是 re_openid
      total_amount,            //后端 query-srv 根据规则函数自动计算得出 
      /// 
      client_ip,               //后端 query-srv route 函数从 前端reqD 中获取
      clientversion$risk_info, //前端 可选填充 
      posttime$risk_info,      //后端 query-srv route 函数 计算当前时间
      /// 
   })=>[null/*cond*/,null/*{reqj,resj}*/]; 
   //------------------------------------------------------------------- 
   query_hb =({                 //redis 自动回调函数使用
       mch_billno                     
   })=>[null/*cond*/,null/*{reqj,resj}*/];
   resend_hb=({             //redis 自动回调函数使用 
       after,                   
       mch_billno,
       re_openid,
       xml_data
   })=>[null/*cond*/,null/*{reqj,resj}*/];
   //
   mobile$risk_info   = null; // risk_info.mobile
   deviceid$risk_info = null; // risk_info.deviceid
   hongbao_total_num  = 1;    
   hongbao_send_name  = null;
   hongbao_wishing    = null;
   hongbao_act_name   = null;
   hongbao_remark     = null;
   hongbao_scene_id   = null; 
   ////----------------------------------------------------------------------
   ////------------------------------------------------------------------------
   async init_wx() {
       /////------------------------------------------------------------
       let {
           wx_app_id_name,
           wx_app_secret_name,
           wx_mch_id_name,
           wx_mch_key_name,
           wx_api_pem_key_name,
           wx_api_pem_cert_name,
           wx_hongbao_send_mobile$risk_info_name,
           wx_hongbao_send_deviceid$risk_info_name,
           wx_hongbao_send_send_name_name,
           wx_hongbao_send_wishing_name,
           wx_hongbao_send_act_name_name,
           wx_hongbao_send_remark_name,
           wx_hongbao_send_scene_id_name,
       } = this.consts;
       //
       let r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_app_id_name}"`});
       let appid   =  r[0].txt;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_app_secret_name}"`});
       let secret  =  r[0].txt;    
       this.appid  = appid;
       this.secret = secret;
       this.code2id = _wxid.creat_code2id(appid,secret);
       //
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_mch_id_name}"`});
       let mchid = r[0].txt;
       this.mchid = mchid;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_mch_key_name}"`});
       this.mchkey = r[0].txt;
       //
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_api_pem_key_name}"`});
       this.api_pem_key = r[0].txt;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_api_pem_cert_name}"`});
       this.api_pem_cert = r[0].txt;
       //
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_mobile$risk_info_name}"`});
       this.mobile$risk_info = r[0].txt;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_deviceid$risk_info_name}"`});
       this.deviceid$risk_info = r[0].txt;
       //*----
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_send_name_name}"`});
       this.hongbao_send_name = r[0].txt;       
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_wishing_name}"`});
       this.hongbao_wishing = r[0].txt;    
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_act_name_name}"`});
       this.hongbao_act_name = r[0].txt;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_remark_name}"`});
       this.hongbao_remark = r[0].txt;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_scene_id_name}"`});
       this.hongbao_scene_id = r[0].txt;   
       //*----
       ////----------------------------------------------------------
       let {
           wx_hongbao_retry_min_intv_name,
           wx_hongbao_send_freq_name,
           wx_hongbao_send_total_num_name,
       } = this.consts;
       ////
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_retry_min_intv_name}"`});
       let hongbao_retry_min_intv   =  r[0].jsn;
       this.hongbao_retry_min_intv  = hongbao_retry_min_intv;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_freq_name}"`});
       let hongbao_send_freq  =  r[0].jsn;       
       this.hongbao_send_freq  = hongbao_send_freq;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_total_num_name}"`});
       let hongbao_total_num = r[0].jsn;
       this.hongbao_total_num = hongbao_total_num;
       //// 
       this.send_hb = _wxhb.creat_send_hb(
           appid,secret,
           this.mchkey,this.api_pem_cert,this.api_pem_key,
           hongbao_retry_min_intv,hongbao_send_freq,
           this.redis.dflt_/*for min intv ctrl*/,
           {
               mch_id:this.mchid,
               mobile$risk_info:   this.mobile$risk_info,
               deviceid$risk_info: this.deviceid$risk_info,
               ////-------------------------------------------    
               total_num:this.hongbao_total_num,
               send_name:this.hongbao_send_name,
               wishing:this.hongbao_wishing,
               act_name:this.hongbao_act_name,
               remark:this.hongbao_remark,
               scene_id:this.hongbao_scene_id,
               ////-------------------------------
           }
       );
       this.query_hb = _wxhb.creat_query_bill(
           appid,secret,
           this.mchkey,this.api_pem_cert,this.api_pem_key,
           hongbao_retry_min_intv,hongbao_send_freq,
           this.redis.dflt_/*for min intv ctrl*/,
           {
               mch_id:this.mchid,
           }
       );
       this.resend_hb = _wxhb.creat_resend_hb(
           appid,secret,
           this.mchkey,this.api_pem_cert,this.api_pem_key,
           hongbao_retry_min_intv,hongbao_send_freq,
           this.redis.dflt_/*for min intv ctrl*/,
       );
       /////
   }
   ////---
   http = null;
   async init_http() {
        let {usock,nginx_user} = this.consts;
        this.http = await _S.creat(usock);
        let Ctor = this.http.constructor;
        add_repr(Ctor,(that)=>that._pipeName);
        _chown_with_name(usock,nginx_user);
   }
   ////---
   handle_dict = {};
   set_handle(route,afunc,reload=false) {
       if(afunc === undefined) {
           let p = _str.trim(route,"/");
           let arr = p.split("/");
           p = arr.join("-");
           if(reload) {
                this.handle_dict[route] = _eng.DFLT_404_HANDLE;
                _rrequire.unrequire("./route/"+p);
           } else {}
           this.handle_dict[route] = require("./route/"+p);
       } else {
           this.handle_dict[route] = afunc;
       }
   } 
   ////---    
   async get_openid(cubody) {
        let {code}        = cubody;
        if(code===undefined) {
            return([false,"code不能为空"])
        } else {
            code = String(code); 
            if(code.length===0) {
                return([false,"code不能为空"])
            } else { 
                let [cond,openid] = await this.code2id(code);
                console.log([cond,openid])
                return([cond,openid])
            }
        }
   }
   //------------  _ 开头的只能被后端使用
   async _send_hongbao(cudata,sign_mode="md5") {
        //{
        //   re_openid      <mandatory,front>,
        //   clientversion  <optional,front>,
        //   total_amount   <mandatory,back query calc>
       //    client_ip      <mandatory,back query add> 
        //}
        //此函数被query模块调用 使用后插入query数据库
        cudata.clientversion$risk_info = cudata.clientversion??'Mobile';
        cudata.posttime$risk_info      = globalThis.CLOCK.clock.dict().mts;
        cudata["#sign_mode"] = sign_mode;
        try{
             //[need_query,need_retry,reason,{jreq,jres}]
             // 
             let rpair = await this.send_hb(cudata);
             return(rpair) 
        } catch(err) {
             err = _err_bsc.to_json(err);   
             return([false,false,err,{jreq,jres}]) 
        }
        //// query-srv 调用此接口后 在redis 插入一个retry 任务 初次等待 30 秒后 查询 
        ////    然后依次*2   60秒 2m 4m 8m 16m 20m  -> jres["#retry"] 中的 after 
        //// 根据查询结果 如果need_retry, 那么到达尝试次数限制后 用 jres["#retry"]
        //   为参数，，调用this._resent_hongbao(jres["#retry"])
       //    requery->requery->requery.....
   }    
   async _query_hongbao(back_data) {
       // 此接口前端不使用
       //{mch_billno}
       let r = await this.query_hb(back_data);
       let need_requery = r["#requery"];
       delete r["#requery"];
       delete r["#chinese"];      // 后端调试使用的中文提示
       return([need_requery,r])   
       // query-srv 调用此接口后
       //    如果need_requery  在redis 插入一个retry query 任务 稳定间隔 30 秒 
       //    否则根据 r 更新数据库
   }
   async _resent_hongbao(retry_cfg_received_data_from_jres) {
       // 此接口前端不使用
       // {after,mch_billno,re_openid,xml_data}
        try{
             //[need_query,need_retry,reason,{jreq,jres}]
             // 
             let rpair = await this.resend_hb(retry_cfg_received_data_from_jres); 
             return(rpair)
        } catch(err) {
             err = _err_bsc.to_json(err);
             return([false,false,err,{jreq,jres}])
        }  
        //// query-srv 调用此接口后 在redis 插入一个retry 任务 初次等待 30 秒后 查询 
        ////    然后依次*2   60秒 2m 4m 8m 16m 20m .... -> jres["#retry"] 中的 after 
        //// 根据查询结果 如果need_retry, 那么用 jres["#retry"]
        //   为参数，，调用this._resent_hongbao(jres["#retry"])
   }
   //// 
   async list_processing() {
       //从 redis.retry_ 中获取  /是那些在SENDING 状态的订单/
       //    调试使用
       //    此接口会很卡
       let retry_redis = redis.retry_; 
       let arr = await retry_redis.list_all();
       return([true,arr])
   }
   //// 
   async change_wx_cfg() {
        try {
            await this.init_wx(); 
            return([true,null])
        } catch(err) {
            return([false,_err_bsc.to_json(err)]) 
        }
   }     
}


const DFLT_CFG = ()=>({
   user:globalThis.USER,
   oss:globalThis.OSS,
   redis:globalThis.REDIS,
   cfgdb:globalThis.CFG,
   validate:_eng.DFLT_VALIDATE,
   ////---- wx_app_id_name  wx_app_secret_name usock nginx_user 
   ..._eng.DFLT_CONSTS(),
   ////
});


async function creat(cfg=DFLT_CFG()) {
    let dflt = DFLT_CFG();
    let ncfg = OASGN(dflt,cfg);
    //--enable service:--
    let srv = new Srv();
    //consts
    {
        let {usock,nginx_user,wx_app_id_name,wx_app_secret_name} = ncfg;  
        srv.init_consts(usock,nginx_user,wx_app_id_name,wx_app_secret_name);  
    }
    //validate
    srv.validate = ncfg.validate;
    // global
    srv.init_glb(ncfg.user,ncfg.oss,ncfg.redis,ncfg.cfgdb);
    // http-server
    srv.init_http();
    // wx
    await srv.init_wx();
    //--add handle  最后接
    for(let route in  _eng.ROUTED()) {  //  route:api_name
       srv.set_handle(route)
    }
    //// --
    srv.http.handler_ = async (req,res) => {
        let D      = await _S.extract_req(req);
        ////--------------------patch-----------------------------
        D.pathname = _url.parse(D.path).pathname;
        ////-----------------------------------------------------
        let afunc = srv.handle_dict[D.path]??srv.handle_dict[D.pathname];
        if(afunc===undefined) {
            return(await _eng.DFLT_ROUTE_NOT_EXIST(D,res,srv))
        } else {
            return(await afunc(D,res,srv));
        }
    }
    //
    return(srv)
}

module.exports = {
   Srv,
   DFLT_CFG, 
   creat, 
}

==> nv-wx-srv-eng-1.0.27.tgz: package/index.js <==
const {OD,OASGN,sign_str,add_repr} = require("nv-facutil-basic");
const _url                         = require("url");
const _str                         = require("nv-string-basic");
const _fbsc                        = require("nv-file-basic");
const _S                           = require("nv-server-simple-stream").http.simple;
const _chown_with_name             = require("nv-cli-user").chown_with_name;
const _rrequire                    = require("nv-facutil-rrequire");
const _eng                         = require("./eng");
const _err_bsc                     = require("nv-error-basic");  
const _wxid                        = require("wx-id");
const _wxhb                        = require("wx-hongbao-pay").func; 



class Srv {
   consts  = _eng.DFLT_CONSTS();
   init_consts(usock,nginx_user,wx_app_id_name,wx_app_secret_name) {
       this.consts.usock        = usock;
       this.consts.nginx_user   = nginx_user; 
       this.wx_app_id_name      = wx_app_id_name;
       this.wx_app_secret_name  = wx_app_secret_name; 
   }
   ////-------------------------------------------------
   user  = null;    //user   globalThis.USER
   oss   = null;    //oss    globalThis.OSS
   redis = null;    //redis  globalThis.REDIS
   cfgdb = null;    //cfg    globalThis.CFG
   //
   init_glb(user,oss,redis,cfgdb) {
        this.user  = user;
        this.oss   = oss;
        this.redis = redis;
        this.cfgdb = cfgdb;
   }
   ////----------------------------------------------
   ////---
   appid        = null; //wxappid
   secret       = null;   
   mchid        = null; //mch_id
   mchkey       = null; //mch_key
   api_pem_key  = null; // internal using
   api_pem_cert = null; // internal using
   code2id = (code)=>code;  
   // 
   hongbao_retry_min_intv = 1000;
   hongbao_send_freq = [30,1000];
   //-----------------发送红包函数签名-------------------------------------- 
   send_hb = ({
      re_openid,               //前端 通过接口get_openid({code}) 获取 openid, 注意这里 openid 名需要是 re_openid
      total_amount,            //后端 query-srv 根据规则函数自动计算得出 
      /// 
      client_ip,               //后端 query-srv route 函数从 前端reqD 中获取
      clientversion$risk_info, //前端 可选填充 
      posttime$risk_info,      //后端 query-srv route 函数 计算当前时间
      /// 
   })=>[null/*cond*/,null/*{reqj,resj}*/]; 
   //------------------------------------------------------------------- 
   query_hb =({                 //redis 自动回调函数使用
       mch_billno                     
   })=>[null/*cond*/,null/*{reqj,resj}*/];
   resend_hb=({             //redis 自动回调函数使用 
       after,                   
       mch_billno,
       re_openid,
       xml_data
   })=>[null/*cond*/,null/*{reqj,resj}*/];
   //
   mobile$risk_info   = null; // risk_info.mobile
   deviceid$risk_info = null; // risk_info.deviceid
   hongbao_total_num  = 1;    
   hongbao_send_name  = null;
   hongbao_wishing    = null;
   hongbao_act_name   = null;
   hongbao_remark     = null;
   hongbao_scene_id   = null; 
   ////----------------------------------------------------------------------
   ////------------------------------------------------------------------------
   async init_wx() {
       /////------------------------------------------------------------
       let {
           wx_app_id_name,
           wx_app_secret_name,
           wx_mch_id_name,
           wx_mch_key_name,
           wx_api_pem_key_name,
           wx_api_pem_cert_name,
           wx_hongbao_send_mobile$risk_info_name,
           wx_hongbao_send_deviceid$risk_info_name,
           wx_hongbao_send_send_name_name,
           wx_hongbao_send_wishing_name,
           wx_hongbao_send_act_name_name,
           wx_hongbao_send_remark_name,
           wx_hongbao_send_scene_id_name,
       } = this.consts;
       //
       let r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_app_id_name}"`});
       let appid   =  r[0].txt;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_app_secret_name}"`});
       let secret  =  r[0].txt;    
       this.appid  = appid;
       this.secret = secret;
       this.code2id = _wxid.creat_code2id(appid,secret);
       //
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_mch_id_name}"`});
       let mchid = r[0].txt;
       this.mchid = mchid;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_mch_key_name}"`});
       this.mchkey = r[0].txt;
       //
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_api_pem_key_name}"`});
       this.api_pem_key = r[0].buf;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_api_pem_cert_name}"`});
       this.api_pem_cert = r[0].buf;
       //
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_mobile$risk_info_name}"`});
       this.mobile$risk_info = r[0].txt;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_deviceid$risk_info_name}"`});
       this.deviceid$risk_info = r[0].txt;
       //*----
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_send_name_name}"`});
       this.hongbao_send_name = r[0].txt;       
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_wishing_name}"`});
       this.hongbao_wishing = r[0].txt;    
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_act_name_name}"`});
       this.hongbao_act_name = r[0].txt;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_remark_name}"`});
       this.hongbao_remark = r[0].txt;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_scene_id_name}"`});
       this.hongbao_scene_id = r[0].txt;   
       //*----
       ////----------------------------------------------------------
       let {
           wx_hongbao_retry_min_intv_name,
           wx_hongbao_send_freq_name,
           wx_hongbao_send_total_num_name,
       } = this.consts;
       ////
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_retry_min_intv_name}"`});
       let hongbao_retry_min_intv   =  r[0].jsn;
       this.hongbao_retry_min_intv  = hongbao_retry_min_intv;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_freq_name}"`});
       let hongbao_send_freq  =  r[0].jsn;       
       this.hongbao_send_freq  = hongbao_send_freq;
       r = await this.cfgdb.select({cond:`seq===0 && name==="${wx_hongbao_send_total_num_name}"`});
       let hongbao_total_num = r[0].jsn;
       this.hongbao_total_num = hongbao_total_num;
       //// 
       this.send_hb = _wxhb.creat_send_hb(
           appid,secret,
           this.mchkey,this.api_pem_cert,this.api_pem_key,
           hongbao_retry_min_intv,hongbao_send_freq,
           this.redis.dflt_/*for min intv ctrl*/,
           {
               mch_id:this.mchid,
               mobile$risk_info:   this.mobile$risk_info,
               deviceid$risk_info: this.deviceid$risk_info,
               ////-------------------------------------------    
               total_num:this.hongbao_total_num,
               send_name:this.hongbao_send_name,
               wishing:this.hongbao_wishing,
               act_name:this.hongbao_act_name,
               remark:this.hongbao_remark,
               scene_id:this.hongbao_scene_id,
               ////-------------------------------
           }
       );
       this.query_hb = _wxhb.creat_query_bill(
           appid,secret,
           this.mchkey,this.api_pem_cert,this.api_pem_key,
           hongbao_retry_min_intv,hongbao_send_freq,
           this.redis.dflt_/*for min intv ctrl*/,
           {
               mch_id:this.mchid,
           }
       );
       this.resend_hb = _wxhb.creat_resend_hb(
           appid,secret,
           this.mchkey,this.api_pem_cert,this.api_pem_key,
           hongbao_retry_min_intv,hongbao_send_freq,
           this.redis.dflt_/*for min intv ctrl*/,
       );
       /////
   }
   ////---
   http = null;
   async init_http() {
        let {usock,nginx_user} = this.consts;
        this.http = await _S.creat(usock);
        let Ctor = this.http.constructor;
        add_repr(Ctor,(that)=>that._pipeName);
        _chown_with_name(usock,nginx_user);
   }
   ////---
   handle_dict = {};
   set_handle(route,afunc,reload=false) {
       if(afunc === undefined) {
           let p = _str.trim(route,"/");
           let arr = p.split("/");
           p = arr.join("-");
           if(reload) {
                this.handle_dict[route] = _eng.DFLT_404_HANDLE;
                _rrequire.unrequire("./route/"+p);
           } else {}
           this.handle_dict[route] = require("./route/"+p);
       } else {
           this.handle_dict[route] = afunc;
       }
   } 
   ////---    
   async get_openid(cubody) {
        let {code}        = cubody;
        if(code===undefined) {
            return([false,"code不能为空"])
        } else {
            code = String(code); 
            if(code.length===0) {
                return([false,"code不能为空"])
            } else { 
                let [cond,openid] = await this.code2id(code);
                console.log([cond,openid])
                return([cond,openid])
            }
        }
   }
   //------------  _ 开头的只能被后端使用
   async _send_hongbao(cudata,sign_mode="md5") {
        //{
        //   re_openid      <mandatory,front>,
        //   clientversion  <optional,front>,
        //   total_amount   <mandatory,back query calc>
       //    client_ip      <mandatory,back query add> 
        //}
        //此函数被query模块调用 使用后插入query数据库
        cudata.clientversion$risk_info = cudata.clientversion??'Mobile';
        cudata.posttime$risk_info      = globalThis.CLOCK.clock.dict().mts;
        cudata["#sign_mode"] = sign_mode;
        try{
             //[need_query,need_retry,reason,{jreq,jres}]
             // 
             let rpair = await this.send_hb(cudata);
             return(rpair) 
        } catch(err) {
             err = _err_bsc.to_json(err);   
             return([false,false,err,{jreq,jres}]) 
        }
        //// query-srv 调用此接口后 在redis 插入一个retry 任务 初次等待 30 秒后 查询 
        ////    然后依次*2   60秒 2m 4m 8m 16m 20m  -> jres["#retry"] 中的 after 
        //// 根据查询结果 如果need_retry, 那么到达尝试次数限制后 用 jres["#retry"]
        //   为参数，，调用this._resent_hongbao(jres["#retry"])
       //    requery->requery->requery.....
   }    
   async _query_hongbao(back_data) {
       // 此接口前端不使用
       //{mch_billno}
       let r = await this.query_hb(back_data);
       let need_requery = r["#requery"];
       delete r["#requery"];
       delete r["#chinese"];      // 后端调试使用的中文提示
       return([need_requery,r])   
       // query-srv 调用此接口后
       //    如果need_requery  在redis 插入一个retry query 任务 稳定间隔 30 秒 
       //    否则根据 r 更新数据库
   }
   async _resent_hongbao(retry_cfg_received_data_from_jres) {
       // 此接口前端不使用
       // {after,mch_billno,re_openid,xml_data}
        try{
             //[need_query,need_retry,reason,{jreq,jres}]
             // 
             let rpair = await this.resend_hb(retry_cfg_received_data_from_jres); 
             return(rpair)
        } catch(err) {
             err = _err_bsc.to_json(err);
             return([false,false,err,{jreq,jres}])
        }  
        //// query-srv 调用此接口后 在redis 插入一个retry 任务 初次等待 30 秒后 查询 
        ////    然后依次*2   60秒 2m 4m 8m 16m 20m .... -> jres["#retry"] 中的 after 
        //// 根据查询结果 如果need_retry, 那么用 jres["#retry"]
        //   为参数，，调用this._resent_hongbao(jres["#retry"])
   }
   //// 
   async list_processing() {
       //从 redis.retry_ 中获取  /是那些在SENDING 状态的订单/
       //    调试使用
       //    此接口会很卡
       let retry_redis = redis.retry_; 
       let arr = await retry_redis.list_all();
       return([true,arr])
   }
   //// 
   async change_wx_cfg() {
        try {
            await this.init_wx(); 
            return([true,null])
        } catch(err) {
            return([false,_err_bsc.to_json(err)]) 
        }
   }     
}


const DFLT_CFG = ()=>({
   user:globalThis.USER,
   oss:globalThis.OSS,
   redis:globalThis.REDIS,
   cfgdb:globalThis.CFG,
   validate:_eng.DFLT_VALIDATE,
   ////---- wx_app_id_name  wx_app_secret_name usock nginx_user 
   ..._eng.DFLT_CONSTS(),
   ////
});


async function creat(cfg=DFLT_CFG()) {
    let dflt = DFLT_CFG();
    let ncfg = OASGN(dflt,cfg);
    //--enable service:--
    let srv = new Srv();
    //consts
    {
        let {usock,nginx_user,wx_app_id_name,wx_app_secret_name} = ncfg;  
        srv.init_consts(usock,nginx_user,wx_app_id_name,wx_app_secret_name);  
    }
    //validate
    srv.validate = ncfg.validate;
    // global
    srv.init_glb(ncfg.user,ncfg.oss,ncfg.redis,ncfg.cfgdb);
    // http-server
    srv.init_http();
    // wx
    await srv.init_wx();
    //--add handle  最后接
    for(let route in  _eng.ROUTED()) {  //  route:api_name
       srv.set_handle(route)
    }
    //// --
    srv.http.handler_ = async (req,res) => {
        let D      = await _S.extract_req(req);
        ////--------------------patch-----------------------------
        D.pathname = _url.parse(D.path).pathname;
        ////-----------------------------------------------------
        let afunc = srv.handle_dict[D.path]??srv.handle_dict[D.pathname];
        if(afunc===undefined) {
            return(await _eng.DFLT_ROUTE_NOT_EXIST(D,res,srv))
        } else {
            return(await afunc(D,res,srv));
        }
    }
    //
    return(srv)
}

module.exports = {
   Srv,
   DFLT_CFG, 
   creat, 
}
